/*******************************************************************************
 * Copyright 2024 ModalAI Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * 4. The Software is used solely in conjunction with devices provided by
 *    ModalAI Inc.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/


#ifndef VOXL_STATE_ESTIMATOR_COMMON_H
#define VOXL_STATE_ESTIMATOR_COMMON_H



// This includes the primary VIO pipe interface. The remainder of this file is
// for things specific to the STATE_PIPE server
#include <modal_pipe_interfaces.h>
#include <modal_pipe_common.h> // for MODAL_PIPE_DEFAULT_BASE_DIR


// config file exists here if you want to access it
#define VOXL_STATE_ESTIMATOR_CONF_FILE	"/etc/modalai/voxl-state-estimator.conf"

#define STATE_PIPE_CONTROL_COMMANDS (RESET_VIO_SOFT "," RESET_VIO_HARD)

#define STATE_PIPE_NAME		"state"
#define STATE_PIPE_LOCATION	MODAL_PIPE_DEFAULT_BASE_DIR STATE_PIPE_NAME "/"

#define LOG_NAME "vse_log"
#define LOG_LOCATION MODAL_PIPE_DEFAULT_BASE_DIR LOG_NAME "/"

typedef struct vft_log_packet{
    uint8_t packet_version;
    int64_t timestamp_ns;

    //timing variables
    int64_t time_pyr;
    int64_t time_extract;
    int64_t time_grid;
    int64_t time_grid2;
    int64_t time_refine;
    int64_t time_estimate;
    int64_t time_lk;
    int64_t time_ransac;
    int64_t time_min_pixel;
    int64_t time_descr;
    int64_t time_total;

    //homography matrix computed by findHomography
    float H00;
    float H01;
    float H02;
    float H10;
    float H11;
    float H12;
    float H20;
    float H21;
    float H22;

    uint32_t pts_last_frame;
    uint32_t pts_post_flow;
    uint32_t pts_post_flowback;
    uint32_t pts_post_ransac;
    uint32_t pts_post_min_pixel;
    uint32_t pts_extracted;

} __attribute((packed))__vft_log_packet;


typedef struct quaternion_ts_t{
    int64_t timestamp_ns;

    float q1; /*<  Quaternion component 1, w (1 in null-rotation)*/
    float q2; /*<  Quaternion component 2, x (0 in null-rotation)*/
    float q3; /*<  Quaternion component 3, y (0 in null-rotation)*/
    float q4; /*<  Quaternion component 4, z (0 in null-rotation)*/
}quaternion_ts_t;

typedef struct vse_log_packet{
    uint8_t packet_version;
    int64_t timestamp_ns;

    float accel_world_x;
    float accel_world_y;
    float accel_world_z;
    int64_t timestamp_ns_accel;

    float of_vx_body_rad;
    float of_vy_body_rad;
    int64_t timestamp_ns_of;

    float vx_of;
    float vy_of;
    float vx_accel;
    float vy_accel;
} __attribute((packed))__vse_log_packet;

#endif // VOXL_STATE_ESTIMATOR_COMMON_H
